@@ -24,18 +24,18 @@ if __name__ == '__main__':
             tests.addTests(loader.discover(start_dir=tests_dir, pattern=pattern))
             return tests
 
-        tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_etree.py"))
-        tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_etree_import.py"))
         tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_helpers.py"))
         tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_namespaces.py"))
+        tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_locations.py"))
         tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_resources.py"))
-        tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_regex.py"))
+        tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_exports.py"))
         tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_xpath.py"))
         tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_cli.py"))
         tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_converters.py"))
         tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_documents.py"))
         tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_dataobjects.py"))
         tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_codegen.py"))
+        tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_translations.py"))
         tests.addTests(loader.discover(start_dir=tests_dir, pattern="test_wsdl.py"))
 
         validation_dir = os.path.join(os.path.dirname(__file__), 'validation')
