@@ -10,18 +10,24 @@
 #
 """Tests on internal helper functions"""
 import unittest
-import sys
 import decimal
+import logging
 from collections import OrderedDict
+from xml.etree import ElementTree
+
+try:
+    import lxml.etree as lxml_etree
+except ImportError:
+    lxml_etree = None
 
 from xmlschema import XMLSchema, XMLSchemaParseError
-from xmlschema.etree import ElementTree, etree_element
 from xmlschema.names import XSD_NAMESPACE, XSI_NAMESPACE, XSD_SCHEMA, \
     XSD_ELEMENT, XSD_SIMPLE_TYPE, XSD_ANNOTATION, XSI_TYPE
 from xmlschema.helpers import prune_etree, get_namespace, get_qname, \
     local_name, get_prefixed_qname, get_extended_qname, raw_xml_encode, \
-    count_digits, strictly_equal
-from xmlschema.testing import iter_nested_items
+    count_digits, strictly_equal, etree_iterpath, etree_getpath, \
+    etree_iter_location_hints, update_namespaces, set_logging_level, logged
+from xmlschema.testing import iter_nested_items, etree_elements_assert_equal
 from xmlschema.validators.exceptions import XMLSchemaValidationError
 from xmlschema.validators.helpers import get_xsd_derivation_attribute, \
     decimal_validator, qname_validator, \
@@ -42,7 +48,7 @@ class TestHelpers(unittest.TestCase):
         XMLSchema.meta_schema.clear()
 
     def test_get_xsd_derivation_attribute(self):
-        elem = etree_element(XSD_ELEMENT, attrib={
+        elem = ElementTree.Element(XSD_ELEMENT, attrib={
             'a1': 'extension', 'a2': ' restriction', 'a3': '#all', 'a4': 'other',
             'a5': 'restriction extension restriction ', 'a6': 'other restriction'
         })
@@ -66,28 +72,28 @@ class TestHelpers(unittest.TestCase):
     def test_parse_component(self):
         component = XMLSchema.meta_schema.types['anyType']
 
-        elem = etree_element(XSD_SCHEMA)
+        elem = ElementTree.Element(XSD_SCHEMA)
         self.assertIsNone(component._parse_child_component(elem))
-        elem.append(etree_element(XSD_ELEMENT))
+        elem.append(ElementTree.Element(XSD_ELEMENT))
         self.assertEqual(component._parse_child_component(elem), elem[0])
-        elem.append(etree_element(XSD_SIMPLE_TYPE))
+        elem.append(ElementTree.Element(XSD_SIMPLE_TYPE))
         self.assertRaises(XMLSchemaParseError, component._parse_child_component, elem)
         self.assertEqual(component._parse_child_component(elem, strict=False), elem[0])
 
         elem.clear()
-        elem.append(etree_element(XSD_ANNOTATION))
+        elem.append(ElementTree.Element(XSD_ANNOTATION))
         self.assertIsNone(component._parse_child_component(elem))
-        elem.append(etree_element(XSD_SIMPLE_TYPE))
+        elem.append(ElementTree.Element(XSD_SIMPLE_TYPE))
         self.assertEqual(component._parse_child_component(elem), elem[1])
-        elem.append(etree_element(XSD_ELEMENT))
+        elem.append(ElementTree.Element(XSD_ELEMENT))
         self.assertRaises(XMLSchemaParseError, component._parse_child_component, elem)
         self.assertEqual(component._parse_child_component(elem, strict=False), elem[1])
 
         elem.clear()
-        elem.append(etree_element(XSD_ANNOTATION))
-        elem.append(etree_element(XSD_ANNOTATION))
+        elem.append(ElementTree.Element(XSD_ANNOTATION))
+        elem.append(ElementTree.Element(XSD_ANNOTATION))
         self.assertIsNone(component._parse_child_component(elem, strict=False))
-        elem.append(etree_element(XSD_SIMPLE_TYPE))
+        elem.append(ElementTree.Element(XSD_SIMPLE_TYPE))
         self.assertEqual(component._parse_child_component(elem), elem[2])
 
     def test_raw_xml_encode_function(self):
@@ -136,9 +142,8 @@ class TestHelpers(unittest.TestCase):
         self.assertFalse(strictly_equal(10, 10.0))
 
     def test_iter_nested_items_function(self):
-        if sys.version_info >= (3, 6):
-            self.assertListEqual(list(iter_nested_items({'a': 10, 'b': 20})), [10, 20])
-            self.assertListEqual(list(iter_nested_items([{'a': 10, 'b': 20}, 30])), [10, 20, 30])
+        self.assertListEqual(list(iter_nested_items({'a': 10, 'b': 20})), [10, 20])
+        self.assertListEqual(list(iter_nested_items([{'a': 10, 'b': 20}, 30])), [10, 20, 30])
 
         with self.assertRaises(TypeError):
             list(iter_nested_items({'a': 10, 'b': 20}, dict_class=OrderedDict))
@@ -275,6 +280,193 @@ class TestHelpers(unittest.TestCase):
         namespaces = {'': XSD_NAMESPACE}
         self.assertEqual(get_extended_qname('element', namespaces), XSD_ELEMENT)
 
+    def test_update_namespaces(self):
+        nsmap = {}
+        update_namespaces(nsmap, [('xs', XSD_NAMESPACE)])
+        self.assertEqual(nsmap, {'xs': XSD_NAMESPACE})
+        update_namespaces(nsmap, [('xs', XSD_NAMESPACE)])
+        self.assertEqual(nsmap, {'xs': XSD_NAMESPACE})
+        update_namespaces(nsmap, [('tns0', 'http://example.com/ns')])
+        self.assertEqual(nsmap, {'xs': XSD_NAMESPACE, 'tns0': 'http://example.com/ns'})
+        update_namespaces(nsmap, [('xs', 'http://example.com/ns')])
+        self.assertEqual(nsmap, {'xs': XSD_NAMESPACE,
+                                 'xs0': 'http://example.com/ns',
+                                 'tns0': 'http://example.com/ns'})
+        update_namespaces(nsmap, [('xs', 'http://example.com/ns')])
+        self.assertEqual(nsmap, {'xs': XSD_NAMESPACE,
+                                 'xs0': 'http://example.com/ns',
+                                 'tns0': 'http://example.com/ns'})
+
+        update_namespaces(nsmap, [('xs', 'http://example.com/ns2')])
+        self.assertEqual(nsmap, {'xs': XSD_NAMESPACE,
+                                 'xs0': 'http://example.com/ns',
+                                 'xs1': 'http://example.com/ns2',
+                                 'tns0': 'http://example.com/ns'})
+
+        nsmap = {}
+        update_namespaces(nsmap, [('', XSD_NAMESPACE)])
+        self.assertEqual(nsmap, {'default': 'http://www.w3.org/2001/XMLSchema'})
+        update_namespaces(nsmap, [('', XSD_NAMESPACE)])
+        self.assertEqual(nsmap, {'default': 'http://www.w3.org/2001/XMLSchema'})
+        update_namespaces(nsmap, [('', 'http://example.com/ns')])
+        self.assertEqual(nsmap, {'default': 'http://www.w3.org/2001/XMLSchema',
+                                 'default0': 'http://example.com/ns'})
+        update_namespaces(nsmap, [('', 'http://example.com/ns2')], root_declarations=True)
+        self.assertEqual(nsmap, {'default': 'http://www.w3.org/2001/XMLSchema',
+                                 'default0': 'http://example.com/ns',
+                                 '': 'http://example.com/ns2'})
+        update_namespaces(nsmap, [('', 'http://example.com/ns2')])
+        self.assertEqual(nsmap, {'default': 'http://www.w3.org/2001/XMLSchema',
+                                 'default0': 'http://example.com/ns',
+                                 '': 'http://example.com/ns2'})
+        update_namespaces(nsmap, [('', 'http://example.com/ns3')])
+        self.assertEqual(nsmap, {'default': 'http://www.w3.org/2001/XMLSchema',
+                                 'default0': 'http://example.com/ns',
+                                 '': 'http://example.com/ns2',
+                                 'default1': 'http://example.com/ns3'})
+
+    def test_etree_iterpath(self):
+        root = ElementTree.XML('<a><b1><c1/><c2/></b1><b2/><b3><c3/></b3></a>')
+
+        items = list(etree_iterpath(root))
+        self.assertListEqual(items, [
+            (root, '.'), (root[0], './b1'), (root[0][0], './b1/c1'),
+            (root[0][1], './b1/c2'), (root[1], './b2'), (root[2], './b3'),
+            (root[2][0], './b3/c3')
+        ])
+
+        self.assertListEqual(items, list(etree_iterpath(root, tag='*')))
+        self.assertListEqual(items, list(etree_iterpath(root, path='')))
+        self.assertListEqual(items, list(etree_iterpath(root, path=None)))
+
+        self.assertListEqual(list(etree_iterpath(root, path='/')), [
+            (root, '/'), (root[0], '/b1'), (root[0][0], '/b1/c1'),
+            (root[0][1], '/b1/c2'), (root[1], '/b2'), (root[2], '/b3'),
+            (root[2][0], '/b3/c3')
+        ])
+
+    def test_etree_getpath(self):
+        root = ElementTree.XML('<a><b1><c1/><c2/></b1><b2/><b3><c3/></b3></a>')
+
+        self.assertEqual(etree_getpath(root, root), '.')
+        self.assertEqual(etree_getpath(root[0], root), './b1')
+        self.assertEqual(etree_getpath(root[2][0], root), './b3/c3')
+        self.assertEqual(etree_getpath(root[0], root, parent_path=True), '.')
+        self.assertEqual(etree_getpath(root[2][0], root, parent_path=True), './b3')
+
+        self.assertIsNone(etree_getpath(root, root[0]))
+        self.assertIsNone(etree_getpath(root[0], root[1]))
+        self.assertIsNone(etree_getpath(root, root, parent_path=True))
+
+    def test_etree_elements_assert_equal(self):
+        e1 = ElementTree.XML('<a><b1>text<c1 a="1"/></b1>\n<b2/><b3/></a>\n')
+        e2 = ElementTree.XML('<a><b1>text<c1 a="1"/></b1>\n<b2/><b3/></a>\n')
+
+        self.assertIsNone(etree_elements_assert_equal(e1, e1))
+        self.assertIsNone(etree_elements_assert_equal(e1, e2))
+
+        if lxml_etree is not None:
+            e2 = lxml_etree.XML('<a><b1>text<c1 a="1"/></b1>\n<b2/><b3/></a>\n')
+            self.assertIsNone(etree_elements_assert_equal(e1, e2))
+
+        e2 = ElementTree.XML('<a><b1>text<c1 a="1"/></b1>\n<b2/><b3/><b4/></a>\n')
+        with self.assertRaises(AssertionError) as ctx:
+            etree_elements_assert_equal(e1, e2)
+        self.assertIn("has lesser children than <Element 'a'", str(ctx.exception))
+
+        e2 = ElementTree.XML('<a><b1>text  <c1 a="1"/></b1>\n<b2/><b3/></a>\n')
+        self.assertIsNone(etree_elements_assert_equal(e1, e2, strict=False))
+        with self.assertRaises(AssertionError) as ctx:
+            etree_elements_assert_equal(e1, e2)
+        self.assertIn("texts differ: 'text' != 'text  '", str(ctx.exception))
+
+        e2 = ElementTree.XML('<a><b1>text<c1 a="1"/></b1>\n<b2>text</b2><b3/></a>\n')
+        with self.assertRaises(AssertionError) as ctx:
+            etree_elements_assert_equal(e1, e2, strict=False)
+        self.assertIn("texts differ: None != 'text'", str(ctx.exception))
+
+        e2 = ElementTree.XML('<a><b1>text<c1 a="1"/></b1>\n<b2/><b3/></a>')
+        self.assertIsNone(etree_elements_assert_equal(e1, e2))
+
+        e2 = ElementTree.XML('<a><b1>text<c1 a="1"/></b1><b2/><b3/></a>\n')
+        self.assertIsNone(etree_elements_assert_equal(e1, e2, strict=False))
+        with self.assertRaises(AssertionError) as ctx:
+            etree_elements_assert_equal(e1, e2)
+        self.assertIn(r"tails differ: '\n' != None", str(ctx.exception))
+
+        e2 = ElementTree.XML('<a><b1>text<c1 a="1 "/></b1>\n<b2/><b3/></a>\n')
+        self.assertIsNone(etree_elements_assert_equal(e1, e2, strict=False))
+        with self.assertRaises(AssertionError) as ctx:
+            etree_elements_assert_equal(e1, e2)
+        self.assertIn("attributes differ: {'a': '1'} != {'a': '1 '}", str(ctx.exception))
+
+        e2 = ElementTree.XML('<a><b1>text<c1 a="2 "/></b1>\n<b2/><b3/></a>\n')
+        with self.assertRaises(AssertionError) as ctx:
+            etree_elements_assert_equal(e1, e2, strict=False)
+        self.assertIn("attribute 'a' values differ: '1' != '2'", str(ctx.exception))
+
+        e2 = ElementTree.XML('<a><!--comment--><b1>text<c1 a="1"/></b1>\n<b2/><b3/></a>\n')
+        self.assertIsNone(etree_elements_assert_equal(e1, e2))
+        self.assertIsNone(etree_elements_assert_equal(e1, e2, skip_comments=False))
+
+        if lxml_etree is not None:
+            e2 = lxml_etree.XML('<a><!--comment--><b1>text<c1 a="1"/></b1>\n<b2/><b3/></a>\n')
+            self.assertIsNone(etree_elements_assert_equal(e1, e2))
+
+        e1 = ElementTree.XML('<a><b1>+1</b1></a>')
+        e2 = ElementTree.XML('<a><b1>+ 1 </b1></a>')
+        self.assertIsNone(etree_elements_assert_equal(e1, e2, strict=False))
+
+        e1 = ElementTree.XML('<a><b1>+1</b1></a>')
+        e2 = ElementTree.XML('<a><b1>+1.1 </b1></a>')
+
+        with self.assertRaises(AssertionError) as ctx:
+            etree_elements_assert_equal(e1, e2, strict=False)
+        self.assertIn("texts differ: '+1' != '+1.1 '", str(ctx.exception))
+
+        e1 = ElementTree.XML('<a><b1>1</b1></a>')
+        e2 = ElementTree.XML('<a><b1>true </b1></a>')
+        self.assertIsNone(etree_elements_assert_equal(e1, e2, strict=False))
+        self.assertIsNone(etree_elements_assert_equal(e2, e1, strict=False))
+
+        e2 = ElementTree.XML('<a><b1>false </b1></a>')
+        with self.assertRaises(AssertionError) as ctx:
+            etree_elements_assert_equal(e1, e2, strict=False)
+        self.assertIn("texts differ: '1' != 'false '", str(ctx.exception))
+
+        e1 = ElementTree.XML('<a><b1> 0</b1></a>')
+        self.assertIsNone(etree_elements_assert_equal(e1, e2, strict=False))
+        self.assertIsNone(etree_elements_assert_equal(e2, e1, strict=False))
+
+        e2 = ElementTree.XML('<a><b1>true </b1></a>')
+        with self.assertRaises(AssertionError) as ctx:
+            etree_elements_assert_equal(e1, e2, strict=False)
+        self.assertIn("texts differ: ' 0' != 'true '", str(ctx.exception))
+
+        e1 = ElementTree.XML('<a><b1>text<c1 a="1"/></b1>\n<b2/><b3/></a>\n')
+        e2 = ElementTree.XML('<a><b1>text<c1 a="1"/>tail</b1>\n<b2/><b3/></a>\n')
+
+        with self.assertRaises(AssertionError) as ctx:
+            etree_elements_assert_equal(e1, e2, strict=False)
+        self.assertIn("tails differ: None != 'tail'", str(ctx.exception))
+
+    def test_iter_location_hints(self):
+        elem = ElementTree.XML(
+            """<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+            xsi:schemaLocation="http://example.com/xmlschema/ns-A import-case4a.xsd"/>"""
+        )
+        self.assertListEqual(
+            list(etree_iter_location_hints(elem)),
+            [('http://example.com/xmlschema/ns-A', 'import-case4a.xsd')]
+        )
+        elem = ElementTree.XML(
+            """<foo xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+            xsi:noNamespaceSchemaLocation="schema.xsd"/>"""
+        )
+        self.assertListEqual(
+            list(etree_iter_location_hints(elem)), [('', 'schema.xsd')]
+        )
+
     def test_prune_etree_function(self):
         root = ElementTree.XML('<A id="0"><B/><C/><D/></A>')
         self.assertFalse(prune_etree(root, lambda x: x.tag == 'C'))
@@ -308,6 +500,14 @@ class TestHelpers(unittest.TestCase):
         self.assertListEqual([e.tag for e in root.iter()], ['A'])
         self.assertEqual(root.attrib, {'id': '1'})
 
+        root = ElementTree.XML('<a><b1><c1/><c2/></b1><b2/><b3><c3/></b3></a>')
+        prune_etree(root, selector=lambda x: x.tag == 'b1')
+        self.assertListEqual([e.tag for e in root.iter()], ['a', 'b2', 'b3', 'c3'])
+
+        root = ElementTree.XML('<a><b1><c1/><c2/></b1><b2/><b3><c3/></b3></a>')
+        prune_etree(root, selector=lambda x: x.tag.startswith('c'))
+        self.assertListEqual([e.tag for e in root.iter()], ['a', 'b1', 'b2', 'b3'])
+
     def test_decimal_validator(self):
         self.assertIsNone(decimal_validator(10))
         self.assertIsNone(decimal_validator(10.1))
@@ -397,6 +597,54 @@ class TestHelpers(unittest.TestCase):
         with self.assertRaises(XMLSchemaValidationError):
             error_type_validator(0)
 
+    def test_set_logging_level(self):
+        logger = logging.getLogger('xmlschema')
+        current_level = logger.level
+        try:
+            self.assertRaises(TypeError, set_logging_level, None)
+            self.assertEqual(logger.level, current_level)
+
+            set_logging_level(logging.DEBUG)
+            self.assertEqual(logger.level, logging.DEBUG)
+
+            set_logging_level('ERROR')
+            self.assertEqual(logger.level, logging.ERROR)
+
+            self.assertRaises(ValueError, set_logging_level, 'WRONG')
+        finally:
+            logger.setLevel(current_level)
+
+    def test_logged_decorator(self):
+        logger = logging.getLogger('xmlschema')
+
+        def func(*args, **kwargs):
+            logger.warning('Warning log line')
+            logger.info('Info log line')
+            logger.debug('Debug log line')
+
+        with self.assertLogs('xmlschema', level='DEBUG') as ctx:
+            logged(func)(loglevel='ERROR')
+            self.assertEqual(logger.level, logging.DEBUG)
+            self.assertEqual(len(ctx.output), 0)
+
+            logged(func)(loglevel='WARNING')
+            self.assertEqual(logger.level, logging.DEBUG)
+            self.assertEqual(len(ctx.output), 1)
+            self.assertIn("Warning log line", ctx.output[-1])
+
+            logged(func)(loglevel='INFO')
+            self.assertEqual(logger.level, logging.DEBUG)
+            self.assertEqual(len(ctx.output), 3)
+            self.assertIn("Warning log line", ctx.output[-2])
+            self.assertIn("Info log line", ctx.output[-1])
+
+            logged(func)()
+            self.assertEqual(logger.level, logging.DEBUG)
+            self.assertEqual(len(ctx.output), 6)
+            self.assertIn("Warning log line", ctx.output[-3])
+            self.assertIn("Info log line", ctx.output[-2])
+            self.assertIn("Debug log line", ctx.output[-1])
+
 
 if __name__ == '__main__':
     import platform
