@@ -12,9 +12,9 @@
 
 import unittest
 import pathlib
+from xml.etree import ElementTree
 
 from xmlschema import XMLSchemaValidationError, XMLSchema10, XMLSchema11
-from xmlschema.etree import ElementTree, ParseError
 from xmlschema.extras.wsdl import WsdlParseError, WsdlComponent, WsdlMessage, \
     WsdlPortType, WsdlOperation, WsdlBinding, WsdlService, Wsdl11Document, \
     WsdlInput, SoapHeader
@@ -95,7 +95,7 @@ WSDL_DOCUMENT_EXAMPLE = """<?xml version="1.0"?>
 
 WSDL_DOCUMENT_NO_SOAP = """<?xml version="1.0"?>
 <wsdl:definitions name="minimal"
-        xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/" 
+        xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
         xmlns:xs="http://www.w3.org/2001/XMLSchema">
 
     <wsdl:message name="myMessage">
@@ -210,7 +210,7 @@ class TestWsdlDocuments(unittest.TestCase):
 
         with self.assertRaises(ValueError) as ctx:
             Wsdl11Document(WSDL_DOCUMENT_EXAMPLE, validation='invalid')
-        self.assertEqual("'invalid': not a validation mode", str(ctx.exception))
+        self.assertEqual("'invalid' is not a validation mode", str(ctx.exception))
 
     def test_example3(self):
         original_example3_file = casepath('features/wsdl/wsdl11_example3.wsdl')
@@ -258,6 +258,19 @@ class TestWsdlDocuments(unittest.TestCase):
         port = wsdl_document.services[service_name].ports['StockQuotePort']
         self.assertEqual(port.soap_location, 'mailto:subscribe@example.com')
 
+    def test_example3_without_types__issue_347(self):
+        no_types_file = casepath('features/wsdl/wsdl11_example3_no_types.wsdl')
+        with self.assertRaises(WsdlParseError):
+            Wsdl11Document(no_types_file)
+
+        schema_file = casepath('features/wsdl/wsdl11_example3_types.xsd')
+        wsdl_document = Wsdl11Document(no_types_file, schema=schema_file)
+
+        self.assertIn('{http://example.com/stockquote.xsd}SubscribeToQuotes',
+                      wsdl_document.schema.maps.elements)
+        self.assertIn('{http://example.com/stockquote.xsd}SubscriptionHeader',
+                      wsdl_document.schema.maps.elements)
+
     def test_example4(self):
         original_example4_file = casepath('features/wsdl/wsdl11_example4.wsdl')
         with self.assertRaises(XMLSchemaValidationError):
@@ -319,7 +332,7 @@ class TestWsdlDocuments(unittest.TestCase):
 
     def test_example5(self):
         original_example5_file = casepath('features/wsdl/wsdl11_example5.wsdl')
-        with self.assertRaises(ParseError):
+        with self.assertRaises(ElementTree.ParseError):
             Wsdl11Document(original_example5_file)
 
         example5_file = casepath('features/wsdl/wsdl11_example5_valid.wsdl')
@@ -401,7 +414,7 @@ class TestWsdlDocuments(unittest.TestCase):
 
     def test_wsdl_document_invalid_imports(self):
         wsdl_template = """<?xml version="1.0"?>
-        <definitions name="import-test1" 
+        <definitions name="import-test1"
                 xmlns="http://schemas.xmlsoap.org/wsdl/">
             <import namespace="http://example.com/ns" location="{0}"/>
         </definitions>"""
@@ -425,7 +438,7 @@ class TestWsdlDocuments(unittest.TestCase):
         self.assertIn('no element found', str(ctx.exception))
 
         wsdl_template = """<?xml version="1.0"?>
-        <definitions name="import-test1" 
+        <definitions name="import-test1"
                 targetNamespace="http://example.com/ns"
                 xmlns="http://schemas.xmlsoap.org/wsdl/">
             <import namespace="http://example.com/ns" location="{0}"/>
@@ -437,7 +450,7 @@ class TestWsdlDocuments(unittest.TestCase):
         self.assertIn('namespace to import must be different', str(ctx.exception))
 
         wsdl_template = """<?xml version="1.0"?>
-        <definitions name="import-test1" 
+        <definitions name="import-test1"
                 targetNamespace="http://example.com/stockquote/definitions"
                 xmlns="http://schemas.xmlsoap.org/wsdl/">
             <import namespace="http://example.com/ns" location="{0}"/>
