@@ -15,6 +15,8 @@ are created using the XSD 1.0 meta-schema or with and additional base schema for
 """
 from decimal import Decimal
 from elementpath import datatypes
+from typing import cast, Any, Dict, Optional, Type, Tuple, Union
+from xml.etree.ElementTree import Element
 
 from ..exceptions import XMLSchemaValueError
 from ..names import XSD_LENGTH, XSD_MIN_LENGTH, XSD_MAX_LENGTH, XSD_ENUMERATION, \
@@ -30,14 +32,14 @@ from ..names import XSD_LENGTH, XSD_MIN_LENGTH, XSD_MAX_LENGTH, XSD_ENUMERATION,
     XSD_DURATION, XSD_DAY_TIME_DURATION, XSD_YEAR_MONTH_DURATION, XSD_BASE64_BINARY, \
     XSD_HEX_BINARY, XSD_NOTATION_TYPE, XSD_ERROR, XSD_ASSERTION, XSD_SIMPLE_TYPE, \
     XSD_ANY_TYPE, XSD_ANY_ATOMIC_TYPE, XSD_ANY_SIMPLE_TYPE
-from ..etree import etree_element
+from ..aliases import ElementType, SchemaType, BaseXsdType
 
 from .helpers import decimal_validator, qname_validator, byte_validator, \
     short_validator, int_validator, long_validator, unsigned_byte_validator, \
     unsigned_short_validator, unsigned_int_validator, unsigned_long_validator, \
     negative_int_validator, positive_int_validator, non_positive_int_validator, \
     non_negative_int_validator, hex_binary_validator, base64_binary_validator, \
-    error_type_validator, boolean_to_python, python_to_boolean
+    error_type_validator, boolean_to_python, python_to_boolean, python_to_float
 from .facets import XSD_10_FACETS_BUILDERS, XSD_11_FACETS_BUILDERS
 from .simple_types import XsdSimpleType, XsdAtomicBuiltin
 
@@ -67,21 +69,20 @@ DATETIME_FACETS = {
     XSD_MIN_EXCLUSIVE, XSD_ASSERTION, XSD_EXPLICIT_TIMEZONE
 }
 
-
 #
 # Element facets instances for builtin types.
-PRESERVE_WHITE_SPACE_ELEMENT = etree_element(XSD_WHITE_SPACE, value='preserve')
-COLLAPSE_WHITE_SPACE_ELEMENT = etree_element(XSD_WHITE_SPACE, value='collapse')
-REPLACE_WHITE_SPACE_ELEMENT = etree_element(XSD_WHITE_SPACE, value='replace')
-XSD10_FLOAT_PATTERN_ELEMENT = etree_element(
+PRESERVE_WHITE_SPACE_ELEMENT = Element(XSD_WHITE_SPACE, value='preserve')
+COLLAPSE_WHITE_SPACE_ELEMENT = Element(XSD_WHITE_SPACE, value='collapse')
+REPLACE_WHITE_SPACE_ELEMENT = Element(XSD_WHITE_SPACE, value='replace')
+XSD10_FLOAT_PATTERN_ELEMENT = Element(
     XSD_PATTERN, value=r"(\+|-)?([0-9]+(\.[0-9]*)?|\.[0-9]+)([Ee](\+|-)?[0-9]+)?|INF|-INF|NaN"
 )
-XSD11_FLOAT_PATTERN_ELEMENT = etree_element(
+XSD11_FLOAT_PATTERN_ELEMENT = Element(
     XSD_PATTERN, value=r"(\+|-)?([0-9]+(\.[0-9]*)?|\.[0-9]+)([Ee](\+|-)?[0-9]+)?|(\+|-)?INF|NaN"
 )
 
 
-XSD_COMMON_BUILTIN_TYPES = (
+XSD_COMMON_BUILTIN_TYPES: Tuple[Dict[str, Any], ...] = (
     # ***********************
     # *** Primitive types ***
     # ***********************
@@ -201,19 +202,19 @@ XSD_COMMON_BUILTIN_TYPES = (
         'name': XSD_LANGUAGE,
         'python_type': str,
         'base_type': XSD_TOKEN,
-        'facets': [etree_element(XSD_PATTERN, value=r"[a-zA-Z]{1,8}(-[a-zA-Z0-9]{1,8})*")]
+        'facets': [Element(XSD_PATTERN, value=r"[a-zA-Z]{1,8}(-[a-zA-Z0-9]{1,8})*")]
     },  # language codes
     {
         'name': XSD_NAME,
         'python_type': str,
         'base_type': XSD_TOKEN,
-        'facets': [etree_element(XSD_PATTERN, value=r"\i\c*")]
+        'facets': [Element(XSD_PATTERN, value=r"\i\c*")]
     },  # not starting with a digit
     {
         'name': XSD_NCNAME,
         'python_type': str,
         'base_type': XSD_NAME,
-        'facets': [etree_element(XSD_PATTERN, value=r"[\i-[:]][\c-[:]]*")]
+        'facets': [Element(XSD_PATTERN, value=r"[\i-[:]][\c-[:]]*")]
     },  # cannot contain colons
     {
         'name': XSD_ID,
@@ -234,7 +235,7 @@ XSD_COMMON_BUILTIN_TYPES = (
         'name': XSD_NMTOKEN,
         'python_type': str,
         'base_type': XSD_TOKEN,
-        'facets': [etree_element(XSD_PATTERN, value=r"\c+")]
+        'facets': [Element(XSD_PATTERN, value=r"\c+")]
     },  # should not contain whitespace (attribute only)
 
     # --- Numerical derived types ---
@@ -248,96 +249,98 @@ XSD_COMMON_BUILTIN_TYPES = (
         'python_type': int,
         'base_type': XSD_INTEGER,
         'facets': [long_validator,
-                   etree_element(XSD_MIN_INCLUSIVE, value='-9223372036854775808'),
-                   etree_element(XSD_MAX_INCLUSIVE, value='9223372036854775807')]
+                   Element(XSD_MIN_INCLUSIVE, value='-9223372036854775808'),
+                   Element(XSD_MAX_INCLUSIVE, value='9223372036854775807')]
     },  # signed 128 bit value
     {
         'name': XSD_INT,
         'python_type': int,
         'base_type': XSD_LONG,
         'facets': [int_validator,
-                   etree_element(XSD_MIN_INCLUSIVE, value='-2147483648'),
-                   etree_element(XSD_MAX_INCLUSIVE, value='2147483647')]
+                   Element(XSD_MIN_INCLUSIVE, value='-2147483648'),
+                   Element(XSD_MAX_INCLUSIVE, value='2147483647')]
     },  # signed 64 bit value
     {
         'name': XSD_SHORT,
         'python_type': int,
         'base_type': XSD_INT,
         'facets': [short_validator,
-                   etree_element(XSD_MIN_INCLUSIVE, value='-32768'),
-                   etree_element(XSD_MAX_INCLUSIVE, value='32767')]
+                   Element(XSD_MIN_INCLUSIVE, value='-32768'),
+                   Element(XSD_MAX_INCLUSIVE, value='32767')]
     },  # signed 32 bit value
     {
         'name': XSD_BYTE,
         'python_type': int,
         'base_type': XSD_SHORT,
         'facets': [byte_validator,
-                   etree_element(XSD_MIN_INCLUSIVE, value='-128'),
-                   etree_element(XSD_MAX_INCLUSIVE, value='127')]
+                   Element(XSD_MIN_INCLUSIVE, value='-128'),
+                   Element(XSD_MAX_INCLUSIVE, value='127')]
     },  # signed 8 bit value
     {
         'name': XSD_NON_NEGATIVE_INTEGER,
         'python_type': int,
         'base_type': XSD_INTEGER,
-        'facets': [non_negative_int_validator, etree_element(XSD_MIN_INCLUSIVE, value='0')]
+        'facets': [non_negative_int_validator, Element(XSD_MIN_INCLUSIVE, value='0')]
     },  # only zero and more value allowed [>= 0]
     {
         'name': XSD_POSITIVE_INTEGER,
         'python_type': int,
         'base_type': XSD_NON_NEGATIVE_INTEGER,
-        'facets': [positive_int_validator, etree_element(XSD_MIN_INCLUSIVE, value='1')]
+        'facets': [positive_int_validator, Element(XSD_MIN_INCLUSIVE, value='1')]
     },  # only positive value allowed [> 0]
     {
         'name': XSD_UNSIGNED_LONG,
         'python_type': int,
         'base_type': XSD_NON_NEGATIVE_INTEGER,
         'facets': [unsigned_long_validator,
-                   etree_element(XSD_MAX_INCLUSIVE, value='18446744073709551615')]
+                   Element(XSD_MAX_INCLUSIVE, value='18446744073709551615')]
     },  # unsigned 128 bit value
     {
         'name': XSD_UNSIGNED_INT,
         'python_type': int,
         'base_type': XSD_UNSIGNED_LONG,
-        'facets': [unsigned_int_validator, etree_element(XSD_MAX_INCLUSIVE, value='4294967295')]
+        'facets': [unsigned_int_validator, Element(XSD_MAX_INCLUSIVE, value='4294967295')]
     },  # unsigned 64 bit value
     {
         'name': XSD_UNSIGNED_SHORT,
         'python_type': int,
         'base_type': XSD_UNSIGNED_INT,
-        'facets': [unsigned_short_validator, etree_element(XSD_MAX_INCLUSIVE, value='65535')]
+        'facets': [unsigned_short_validator, Element(XSD_MAX_INCLUSIVE, value='65535')]
     },  # unsigned 32 bit value
     {
         'name': XSD_UNSIGNED_BYTE,
         'python_type': int,
         'base_type': XSD_UNSIGNED_SHORT,
-        'facets': [unsigned_byte_validator, etree_element(XSD_MAX_INCLUSIVE, value='255')]
+        'facets': [unsigned_byte_validator, Element(XSD_MAX_INCLUSIVE, value='255')]
     },  # unsigned 8 bit value
     {
         'name': XSD_NON_POSITIVE_INTEGER,
         'python_type': int,
         'base_type': XSD_INTEGER,
-        'facets': [non_positive_int_validator, etree_element(XSD_MAX_INCLUSIVE, value='0')]
+        'facets': [non_positive_int_validator, Element(XSD_MAX_INCLUSIVE, value='0')]
     },  # only zero and smaller value allowed [<= 0]
     {
         'name': XSD_NEGATIVE_INTEGER,
         'python_type': int,
         'base_type': XSD_NON_POSITIVE_INTEGER,
-        'facets': [negative_int_validator, etree_element(XSD_MAX_INCLUSIVE, value='-1')]
+        'facets': [negative_int_validator, Element(XSD_MAX_INCLUSIVE, value='-1')]
     },  # only negative value allowed [< 0]
 )
 
-XSD_10_BUILTIN_TYPES = XSD_COMMON_BUILTIN_TYPES + (
+XSD_10_BUILTIN_TYPES: Tuple[Dict[str, Any], ...] = XSD_COMMON_BUILTIN_TYPES + (
     {
         'name': XSD_DOUBLE,
         'python_type': float,
         'admitted_facets': FLOAT_FACETS,
         'facets': [XSD10_FLOAT_PATTERN_ELEMENT, COLLAPSE_WHITE_SPACE_ELEMENT],
+        'from_python': python_to_float,
     },  # 64 bit floating point
     {
         'name': XSD_FLOAT,
         'python_type': float,
         'admitted_facets': FLOAT_FACETS,
         'facets': [XSD10_FLOAT_PATTERN_ELEMENT, COLLAPSE_WHITE_SPACE_ELEMENT],
+        'from_python': python_to_float,
     },  # 32 bit floating point
 
     # --- Year related primitive types (year 0 not allowed) ---
@@ -371,18 +374,20 @@ XSD_10_BUILTIN_TYPES = XSD_COMMON_BUILTIN_TYPES + (
     },  # [-][Y*]YYYY-MM
 )
 
-XSD_11_BUILTIN_TYPES = XSD_COMMON_BUILTIN_TYPES + (
+XSD_11_BUILTIN_TYPES: Tuple[Dict[str, Any], ...] = XSD_COMMON_BUILTIN_TYPES + (
     {
         'name': XSD_DOUBLE,
         'python_type': float,
         'admitted_facets': FLOAT_FACETS,
         'facets': [XSD11_FLOAT_PATTERN_ELEMENT, COLLAPSE_WHITE_SPACE_ELEMENT],
+        'from_python': python_to_float,
     },  # 64 bit floating point
     {
         'name': XSD_FLOAT,
         'python_type': float,
         'admitted_facets': FLOAT_FACETS,
         'facets': [XSD11_FLOAT_PATTERN_ELEMENT, COLLAPSE_WHITE_SPACE_ELEMENT],
+        'from_python': python_to_float,
     },  # 32 bit floating point
 
     # --- Year related primitive types (year 0 allowed and mapped to 1 BCE) ---
@@ -420,14 +425,14 @@ XSD_11_BUILTIN_TYPES = XSD_COMMON_BUILTIN_TYPES + (
         'python_type': (datatypes.DateTimeStamp, str),
         'base_type': XSD_DATETIME,
         'to_python': datatypes.DateTime.fromstring,
-        'facets': [etree_element(XSD_EXPLICIT_TIMEZONE, value='required')],
+        'facets': [Element(XSD_EXPLICIT_TIMEZONE, value='required')],
     },  # [-][Y*]YYYY-MM-DD[Thh:mm:ss] with required timezone
     {
         'name': XSD_DAY_TIME_DURATION,
         'python_type': (datatypes.DayTimeDuration, str),
         'base_type': XSD_DURATION,
         'to_python': datatypes.DayTimeDuration.fromstring,
-    },  # PnYnMnDTnHnMnS with month an year equal to 0
+    },  # PnYnMnDTnHnMnS with month a year equal to 0
     {
         'name': XSD_YEAR_MONTH_DURATION,
         'python_type': (datatypes.YearMonthDuration, str),
@@ -444,7 +449,10 @@ XSD_11_BUILTIN_TYPES = XSD_COMMON_BUILTIN_TYPES + (
 )
 
 
-def xsd_builtin_types_factory(meta_schema, xsd_types, atomic_builtin_class=None):
+def xsd_builtin_types_factory(
+        meta_schema: SchemaType,
+        xsd_types: Dict[str, Union[BaseXsdType, Tuple[ElementType, SchemaType]]],
+        atomic_builtin_class: Optional[Type[XsdAtomicBuiltin]] = None) -> None:
     """
     Builds the dictionary for XML Schema built-in types mapping.
     """
@@ -465,8 +473,8 @@ def xsd_builtin_types_factory(meta_schema, xsd_types, atomic_builtin_class=None)
 
     # xs:anySimpleType
     # Ref: https://www.w3.org/TR/xmlschema11-2/#builtin-stds
-    xsd_types[XSD_ANY_SIMPLE_TYPE] = XsdSimpleType(
-        elem=etree_element(XSD_SIMPLE_TYPE, name=XSD_ANY_SIMPLE_TYPE),
+    xsd_any_simple_type = xsd_types[XSD_ANY_SIMPLE_TYPE] = XsdSimpleType(
+        elem=Element(XSD_SIMPLE_TYPE, name=XSD_ANY_SIMPLE_TYPE),
         schema=meta_schema,
         parent=None,
         name=XSD_ANY_SIMPLE_TYPE
@@ -474,39 +482,41 @@ def xsd_builtin_types_factory(meta_schema, xsd_types, atomic_builtin_class=None)
 
     # xs:anyAtomicType
     # Ref: https://www.w3.org/TR/xmlschema11-2/#builtin-stds
-    xsd_types[XSD_ANY_ATOMIC_TYPE] = meta_schema.BUILDERS.restriction_class(
-        elem=etree_element(XSD_SIMPLE_TYPE, name=XSD_ANY_ATOMIC_TYPE),
+    xsd_types[XSD_ANY_ATOMIC_TYPE] = meta_schema.xsd_atomic_restriction_class(
+        elem=Element(XSD_SIMPLE_TYPE, name=XSD_ANY_ATOMIC_TYPE),
         schema=meta_schema,
         parent=None,
         name=XSD_ANY_ATOMIC_TYPE,
-        base_type=xsd_types[XSD_ANY_SIMPLE_TYPE]
+        base_type=xsd_any_simple_type,
     )
 
     for item in builtin_types:
         item = item.copy()
-        name = item['name']
+        name: str = item['name']
         try:
-            elem, schema = xsd_types[name]
+            value = cast(Tuple[ElementType, SchemaType], xsd_types[name])
         except KeyError:
             # If builtin type element is missing create a dummy element. Necessary for the
             # meta-schema XMLSchema.xsd of XSD 1.1, that not includes builtins declarations.
-            elem = etree_element(XSD_SIMPLE_TYPE, name=name, id=name)
+            elem = Element(XSD_SIMPLE_TYPE, name=name, id=name)
         else:
+            elem, schema = value
             if schema is not meta_schema:
                 raise XMLSchemaValueError("loaded entry schema is not the meta-schema!")
 
+        base_type: Union[None, BaseXsdType, Tuple[ElementType, SchemaType]]
         if 'base_type' in item:
             base_type = item['base_type'] = xsd_types[item['base_type']]
         else:
             base_type = None
 
         facets = item.pop('facets', None)
-        builtin_type = atomic_builtin_class(elem, meta_schema, **item)
+        builtin_type: XsdAtomicBuiltin = atomic_builtin_class(elem, meta_schema, **item)
         if facets:
             built_facets = builtin_type.facets
             for e in facets:
                 try:
-                    cls = facets_map[e.tag]
+                    cls: Any = facets_map[e.tag]
                 except AttributeError:
                     built_facets[None] = e
                 else:
