@@ -10,20 +10,28 @@
 """
 This module contains classes for other XML Schema identity constraints.
 """
+import copy
 import re
 import math
-from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Pattern, \
+from typing import TYPE_CHECKING, cast, Any, Dict, Iterator, List, Optional, Pattern, \
     Tuple, Union, Counter
-from elementpath import XPath2Parser, ElementPathError, XPathToken, XPathContext, \
-    translate_pattern, datatypes
+
+from elementpath import ElementPathError, XPathContext, \
+    ElementNode, translate_pattern, AttributeNode
+from elementpath.datatypes import UntypedAtomic
 
 from ..exceptions import XMLSchemaTypeError, XMLSchemaValueError
-from ..names import XSD_QNAME, XSD_UNIQUE, XSD_KEY, XSD_KEYREF, XSD_SELECTOR, XSD_FIELD
+from ..names import XSD_UNIQUE, XSD_KEY, XSD_KEYREF, XSD_SELECTOR, XSD_FIELD
+from ..translation import gettext as _
 from ..helpers import get_qname, get_extended_qname
-from ..aliases import ElementType, SchemaType, NamespacesType, AtomicValueType
-from ..xpath import iter_schema_nodes
+from ..aliases import ElementType, SchemaType, NamespacesType, AtomicValueType, \
+    BaseXsdType, SchemaElementType, SchemaAttributeType
+from ..xpath import IdentityXPathParser, XPathElement
+from .exceptions import XMLSchemaNotBuiltError
 from .xsdbase import XsdComponent
 from .attributes import XsdAttribute
+from .wildcards import XsdAnyElement, XsdWildcard
+from . import elements as elements_module
 
 if TYPE_CHECKING:
     from .elements import XsdElement
@@ -32,31 +40,8 @@ IdentityFieldItemType = Union[AtomicValueType, XsdAttribute, Tuple[Any, ...], No
 IdentityCounterType = Tuple[IdentityFieldItemType, ...]
 IdentityMapType = Dict[Union['XsdKey', 'XsdKeyref', str, None],
                        Union['IdentityCounter', 'KeyrefCounter']]
-
-XSD_IDENTITY_XPATH_SYMBOLS = frozenset((
-    'processing-instruction', 'following-sibling', 'preceding-sibling',
-    'ancestor-or-self', 'attribute', 'following', 'namespace', 'preceding',
-    'ancestor', 'position', 'comment', 'parent', 'child', 'false', 'text', 'node',
-    'true', 'last', 'not', 'and', 'mod', 'div', 'or', '..', '//', '!=', '<=', '>=',
-    '(', ')', '[', ']', '.', '@', ',', '/', '|', '*', '-', '=', '+', '<', '>', ':',
-    '(end)', '(unknown)', '(invalid)', '(name)', '(string)', '(float)', '(decimal)',
-    '(integer)', '::', '{', '}',
-))
-
-
-# XSD identities use a restricted parser and a context for iterate element
-# references. The XMLSchemaProxy is not used for the specific selection of
-# fields and elements and the XSD fields are got at first validation run.
-class IdentityXPathContext(XPathContext):
-    _iter_nodes = staticmethod(iter_schema_nodes)
-
-
-class IdentityXPathParser(XPath2Parser):
-    symbol_table = {
-        k: v for k, v in XPath2Parser.symbol_table.items()  # type: ignore[misc]
-        if k in XSD_IDENTITY_XPATH_SYMBOLS
-    }
-    SYMBOLS = XSD_IDENTITY_XPATH_SYMBOLS
+IdentityNodeType = Union[ElementNode, AttributeNode]
+FieldDecoderType = Union[SchemaElementType, SchemaAttributeType]
 
 
 class XsdSelector(XsdComponent):
@@ -71,18 +56,16 @@ class XsdSelector(XsdComponent):
         lazy_quantifiers=False,
         anchors=False
     )
-    token = None   # type: XPathToken
-    parser = None  # type: IdentityXPathParser
 
     def __init__(self, elem: ElementType, schema: SchemaType,
                  parent: Optional['XsdIdentity']) -> None:
-        super(XsdSelector, self).__init__(elem, schema, parent)
+        super().__init__(elem, schema, parent)
 
     def _parse(self) -> None:
         try:
             self.path = self.elem.attrib['xpath']
         except KeyError:
-            self.parse_error("'xpath' attribute required")
+            self.parse_error(_("'xpath' attribute required"))
             self.path = '*'
         else:
             path = self.path.replace(' ', '')
@@ -94,7 +77,7 @@ class XsdSelector(XsdComponent):
                 _match = self.pattern.match(path)  # type: ignore[union-attr]
 
             if not _match:
-                msg = "invalid XPath expression for an {}"
+                msg = _("invalid XPath expression for an {}")
                 self.parse_error(msg.format(self.__class__.__name__))
 
         # XSD 1.1 xpathDefaultNamespace attribute
@@ -122,18 +105,7 @@ class XsdSelector(XsdComponent):
 
     @property
     def built(self) -> bool:
-        return self.token is not None
-
-    @property
-    def target_namespace(self) -> str:
-        # TODO: implement a property in elementpath for getting XPath token's namespace
-        if self.token is None:
-            pass  # xpathDefaultNamespace="##targetNamespace"
-        elif self.token.symbol == ':':
-            return self.token[1].namespace or self.xpath_default_namespace
-        elif self.token.symbol == '@' and self.token[0].symbol == ':':
-            return self.token[0][1].namespace or self.xpath_default_namespace
-        return self.schema.target_namespace
+        return True
 
 
 class XsdFieldSelector(XsdSelector):
@@ -164,21 +136,21 @@ class XsdIdentity(XsdComponent):
     parent: 'XsdElement'
     ref: Optional['XsdIdentity']
 
-    selector = None  # type: XsdSelector
-    fields = ()      # type: Union[Tuple[()], List[XsdFieldSelector]]
+    selector: Optional[XsdSelector] = None
+    fields: List[XsdFieldSelector]
 
     # XSD elements bound by selector (for speed-up and for lazy mode)
-    elements: Union[Tuple[()], Dict['XsdElement', Optional[IdentityCounterType]]] = ()
+    elements: Dict['XsdElement', List['FieldValueSelector']]
 
     def __init__(self, elem: ElementType, schema: SchemaType,
                  parent: Optional['XsdElement']) -> None:
-        super(XsdIdentity, self).__init__(elem, schema, parent)
+        super().__init__(elem, schema, parent)
 
     def _parse(self) -> None:
         try:
             self.name = get_qname(self.target_namespace, self.elem.attrib['name'])
         except KeyError:
-            self.parse_error("missing required attribute 'name'")
+            self.parse_error(_("missing required attribute 'name'"))
             self.name = ''
 
         for child in self.elem:
@@ -186,7 +158,7 @@ class XsdIdentity(XsdComponent):
                 self.selector = XsdSelector(child, self.schema, self)
                 break
         else:
-            self.parse_error("missing 'selector' declaration.")
+            self.parse_error(_("missing 'selector' declaration"))
 
         self.fields = []
         for child in self.elem:
@@ -198,131 +170,67 @@ class XsdIdentity(XsdComponent):
             try:
                 ref = self.maps.identities[self.name]
             except KeyError:
-                self.parse_error("unknown identity constraint {!r}".format(self.name))
+                msg = _("unknown identity constraint {!r}")
+                self.parse_error(msg.format(self.name))
                 return
             else:
                 if not isinstance(ref, self.__class__):
-                    self.parse_error("attribute 'ref' points to a different kind constraint")
+                    msg = _("attribute 'ref' points to a different kind constraint")
+                    self.parse_error(msg)
                 self.selector = ref.selector
                 self.fields = ref.fields
                 self.ref = ref
 
-        context = IdentityXPathContext(self.schema, item=self.parent)  # type: ignore
+        if self.selector is None:
+            return  # Do not raise, already found by meta-schema validation.
 
-        self.elements = {}
-        try:
-            for e in self.selector.token.select_results(context):
-                if not isinstance(e, XsdComponent) or isinstance(e, XsdAttribute):
-                    self.parse_error("selector xpath expression can only select elements")
-                elif e.name is not None:
-                    if TYPE_CHECKING:
-                        assert isinstance(e, XsdElement)  # for mypy checks with Python 3.7
-                    self.elements[e] = None
-        except AttributeError:
-            pass
-        else:
-            if not self.elements:
-                # Try to detect target XSD elements extracting QNames
-                # of the leaf elements from the XPath expression and
-                # use them to match global elements.
+        self.elements = self.get_selected_elements(base_element=self.parent)
 
-                qname: Any
-                for qname in self.selector.token.iter_leaf_elements():
-                    xsd_element = self.maps.elements.get(
-                        get_extended_qname(qname, self.namespaces)
-                    )
-                    if xsd_element is not None and \
-                            not isinstance(xsd_element, tuple) and \
-                            xsd_element not in self.elements:
-                        self.elements[xsd_element] = None
+    def get_selected_elements(self, base_element: Union['XsdElement', XPathElement]) \
+            -> Dict['XsdElement', List['FieldValueSelector']]:
+        elements: Dict['XsdElement', List['FieldValueSelector']] = {}
+        if self.selector is None:
+            return elements
+
+        context = XPathContext(self.schema.xpath_node, item=base_element.xpath_node)
+        for e in self.selector.token.select_results(context):
+            if isinstance(e, elements_module.XsdElement):
+                if e.name is not None:
+                    if e.ref is not None:
+                        e = e.ref
+                    if e not in elements:
+                        elements[e] = [FieldValueSelector(f, e) for f in self.fields]
+                        e.selected_by.add(self)
+
+            elif not isinstance(e, XsdAnyElement):
+                msg = _("selector xpath expression can only select elements")
+                self.parse_error(msg)
+
+        if not elements:
+            # Try to detect target XSD elements extracting QNames
+            # of the leaf elements from the XPath expression and
+            # use them to match global elements.
+
+            qname: Any
+            for qname in self.selector.token.iter_leaf_elements():
+                e1 = self.maps.elements.get(
+                    get_extended_qname(qname, self.namespaces)
+                )
+                if e1 is not None and not isinstance(e1, tuple) and e1 not in elements:
+                    if e1.ref is not None:
+                        e1 = e1.ref
+                    if e1 not in elements:
+                        elements[e1] = [FieldValueSelector(f, e1) for f in self.fields]
+                        e1.selected_by.add(self)
+
+        return elements
 
     @property
     def built(self) -> bool:
-        return not isinstance(self.elements, tuple)
+        return 'elements' in self.__dict__
 
-    def get_fields(self, elem: Union[ElementType, 'XsdElement'],
-                   namespaces: Optional[NamespacesType] = None,
-                   decoders: Optional[Tuple[XsdAttribute, ...]] = None) -> IdentityCounterType:
-        """
-        Get fields for a schema or instance context element.
-
-        :param elem: an Element or an XsdElement
-        :param namespaces: is an optional mapping from namespace prefix to URI.
-        :param decoders: context schema fields decoders.
-        :return: a tuple with field values. An empty field is replaced by `None`.
-        """
-        fields: List[IdentityFieldItemType] = []
-
-        if not isinstance(elem, XsdComponent):
-            context_class = XPathContext
-        else:
-            context_class = IdentityXPathContext
-
-        result: Any
-        value: Union[AtomicValueType, None]
-        for k, field in enumerate(self.fields):
-            result = field.token.get_results(context_class(elem))  # type: ignore
-
-            if not result:
-                if decoders is not None and decoders[k] is not None:
-                    value = decoders[k].value_constraint
-                    if value is not None:
-                        if decoders[k].type.root_type.name == XSD_QNAME:
-                            value = get_extended_qname(value, namespaces)
-
-                        if isinstance(value, list):
-                            fields.append(tuple(value))
-                        elif isinstance(value, bool):
-                            fields.append((value, bool))
-                        elif not isinstance(value, float):
-                            fields.append(value)
-                        elif math.isnan(value):
-                            fields.append(('nan', float))
-                        else:
-                            fields.append((value, float))
-
-                        continue
-
-                if not isinstance(self, XsdKey) or 'ref' in elem.attrib and \
-                        self.schema.meta_schema is None and self.schema.XSD_VERSION != '1.0':
-                    fields.append(None)
-                elif field.target_namespace not in self.maps.namespaces:
-                    fields.append(None)
-                else:
-                    msg = "missing key field {!r} for {!r}"
-                    raise XMLSchemaValueError(msg.format(field.path, self))
-
-            elif len(result) == 1:
-                if decoders is None or decoders[k] is None:
-                    fields.append(result[0])
-                else:
-                    if decoders[k].type.content_type_label not in ('simple', 'mixed'):
-                        raise XMLSchemaTypeError("%r field doesn't have a simple type!" % field)
-
-                    value = decoders[k].data_value(result[0])
-                    if decoders[k].type.root_type.name == XSD_QNAME:
-                        if isinstance(value, str):
-                            value = get_extended_qname(value, namespaces)
-                        elif isinstance(value, datatypes.QName):
-                            value = value.expanded_name
-
-                    if isinstance(value, list):
-                        fields.append(tuple(value))
-                    elif isinstance(value, bool):
-                        fields.append((value, bool))
-                    elif not isinstance(value, float):
-                        fields.append(value)
-                    elif math.isnan(value):
-                        fields.append(('nan', float))
-                    else:
-                        fields.append((value, float))
-            else:
-                raise XMLSchemaValueError("%r field selects multiple values!" % field)
-
-        return tuple(fields)
-
-    def get_counter(self, enabled: bool = True) -> 'IdentityCounter':
-        return IdentityCounter(self, enabled)
+    def get_counter(self, elem: ElementType) -> 'IdentityCounter':
+        return IdentityCounter(self, elem)
 
 
 class XsdUnique(XsdIdentity):
@@ -345,17 +253,17 @@ class XsdKeyref(XsdIdentity):
     refer_path = '.'
 
     def _parse(self) -> None:
-        super(XsdKeyref, self)._parse()
+        super()._parse()
         try:
             self.refer = self.schema.resolve_qname(self.elem.attrib['refer'])
         except (KeyError, ValueError, RuntimeError) as err:
             if 'refer' not in self.elem.attrib:
-                self.parse_error("missing required attribute 'refer'")
+                self.parse_error(_("missing required attribute 'refer'"))
             else:
                 self.parse_error(err)
 
     def build(self) -> None:
-        super(XsdKeyref, self).build()
+        super().build()
 
         if isinstance(self.refer, (XsdKey, XsdUnique)):
             return  # referenced key/unique identity constraint already set
@@ -365,20 +273,29 @@ class XsdKeyref(XsdIdentity):
         if self.refer is None:
             return  # attribute or key/unique identity constraint missing
         elif isinstance(self.refer, str):
-            refer = self.parent.identities.get(self.refer)
+            refer: Optional[XsdIdentity]
+            for refer in self.parent.identities:
+                if refer.name == self.refer:
+                    break
+            else:
+                refer = None
+
             if refer is not None and refer.ref is None:
                 self.refer = refer  # type: ignore[assignment]
             else:
                 try:
                     self.refer = self.maps.identities[self.refer]  # type: ignore[assignment]
                 except KeyError:
-                    self.parse_error("key/unique identity constraint %r is missing" % self.refer)
+                    msg = _("key/unique identity constraint %r is missing")
+                    self.parse_error(msg % self.refer)
                     return
 
         if not isinstance(self.refer, (XsdKey, XsdUnique)):
-            self.parse_error("reference to a non key/unique identity constraint %r" % self.refer)
+            msg = _("reference to a non key/unique identity constraint %r")
+            self.parse_error(msg % self.refer)
         elif len(self.refer.fields) != len(self.fields):
-            self.parse_error("field cardinality mismatch between %r and %r" % (self, self.refer))
+            msg = _("field cardinality mismatch between {0!r} and {1!r}")
+            self.parse_error(msg.format(self, self.refer))
         elif self.parent is not self.refer.parent:
             refer_path = self.refer.parent.get_path(ancestor=self.parent)
             if refer_path is None:
@@ -402,8 +319,8 @@ class XsdKeyref(XsdIdentity):
     def built(self) -> bool:
         return not isinstance(self.elements, tuple) and isinstance(self.refer, XsdIdentity)
 
-    def get_counter(self, enabled: bool = True) -> 'KeyrefCounter':
-        return KeyrefCounter(self, enabled)
+    def get_counter(self, elem: ElementType) -> 'KeyrefCounter':
+        return KeyrefCounter(self, elem)
 
 
 class Xsd11Unique(XsdUnique):
@@ -411,7 +328,7 @@ class Xsd11Unique(XsdUnique):
         if self._parse_reference():
             self.ref = True  # type: ignore[assignment]
         else:
-            super(Xsd11Unique, self)._parse()
+            super()._parse()
 
 
 class Xsd11Key(XsdKey):
@@ -419,7 +336,7 @@ class Xsd11Key(XsdKey):
         if self._parse_reference():
             self.ref = True  # type: ignore[assignment]
         else:
-            super(Xsd11Key, self)._parse()
+            super()._parse()
 
 
 class Xsd11Keyref(XsdKeyref):
@@ -427,27 +344,31 @@ class Xsd11Keyref(XsdKeyref):
         if self._parse_reference():
             self.ref = True  # type: ignore[assignment]
         else:
-            super(Xsd11Keyref, self)._parse()
+            super()._parse()
 
 
 class IdentityCounter:
 
-    def __init__(self, identity: XsdIdentity, enabled: bool = True) -> None:
+    def __init__(self, identity: XsdIdentity, elem: ElementType) -> None:
         self.counter: Counter[IdentityCounterType] = Counter[IdentityCounterType]()
         self.identity = identity
-        self.enabled = enabled
+        self.elem = elem
+        self.enabled = True
+        self.elements = None
 
     def __repr__(self) -> str:
         return "%s%r" % (self.__class__.__name__[:-7], self.counter)
 
-    def clear(self) -> None:
+    def reset(self, elem: ElementType) -> None:
         self.counter.clear()
+        self.elem = elem
         self.enabled = True
+        self.elements = None
 
     def increase(self, fields: IdentityCounterType) -> None:
         self.counter[fields] += 1
         if self.counter[fields] == 2:
-            msg = "duplicated value {!r} for {!r}"
+            msg = _("duplicated value {0!r} for {1!r}")
             raise XMLSchemaValueError(msg.format(fields, self.identity))
 
 
@@ -469,3 +390,121 @@ class KeyrefCounter(IdentityCounter):
             else:
                 msg = "value {} not found for {!r}"
                 yield XMLSchemaValueError(msg.format(v, self.identity.refer))
+
+
+class FieldValueSelector:
+
+    skip_wildcard = False
+
+    def __init__(self, field: XsdFieldSelector, xsd_element: 'XsdElement') -> None:
+        if field.token is None:
+            msg = f"identity field {field} is not built"
+            raise XMLSchemaNotBuiltError(field, msg)
+
+        self.field = field
+        self.xsd_element = xsd_element
+        self.value_constraints = {}
+
+        self.token = copy.deepcopy(field.token)
+        schema_context = xsd_element.xpath_proxy.get_context()
+        self.decoders = []
+
+        for node in self.token.select(schema_context):
+            if not isinstance(node, (AttributeNode, ElementNode)):
+                raise XMLSchemaTypeError(
+                    "xs:field path must select only attributes and elements"
+                )
+
+            comp = cast(FieldDecoderType, node.value)
+            self.decoders.append(comp)
+            if isinstance(comp, XsdWildcard):
+                if comp.process_contents == 'skip':
+                    self.skip_wildcard = True
+            else:
+                value_constraint = comp.value_constraint
+                if value_constraint is not None:
+                    self.value_constraints[node.name] = comp.type.text_decode(value_constraint)
+                    if isinstance(comp, XsdAttribute):
+                        self.value_constraints[None] = self.value_constraints[node.name]
+
+        if len(self.decoders) > 1 and None in self.value_constraints:
+            self.value_constraints.pop(None)
+
+    def get_value(self, element_node: ElementNode,
+                  namespaces: Optional[NamespacesType] = None) -> IdentityFieldItemType:
+        """
+        Get field value from an element node for a schema or instance context element.
+
+        :param element_node: a no Element
+        :param namespaces: is an optional mapping from namespace prefix to URI.
+        """
+        value: Union[AtomicValueType, List[AtomicValueType], None] = None
+        context = XPathContext(element_node, namespaces=namespaces)
+        elem = element_node.elem  # type: ignore[attr-defined, unused-ignore]
+
+        empty = True
+        for node in cast(Iterator[IdentityNodeType], self.token.select(context)):
+            if empty:
+                empty = False
+            else:
+                msg = _("%r field selects multiple values!")
+                raise XMLSchemaValueError(msg % self.field)
+
+            try:
+                xsd_type = cast(Optional[BaseXsdType], node.xsd_type)
+            except AttributeError:
+                msg = _("%r field selects a %r!")
+                raise XMLSchemaTypeError(msg % (self.field, type(node)))
+
+            if xsd_type is None:
+                if self.skip_wildcard:
+                    value = None
+                else:
+                    value = node.string_value
+            elif xsd_type.content_type_label not in ('simple', 'mixed'):
+                msg = _("%r field doesn't have a simple type!")
+                raise XMLSchemaTypeError(msg % self.field)
+            elif xsd_type.is_qname():
+                value = get_extended_qname(node.string_value.strip(), namespaces)
+            elif xsd_type.is_boolean():
+                # Workarounds for discovered issues with XPath processors
+                value = xsd_type.text_decode(node.string_value.strip())
+            else:
+                try:
+                    value = node.typed_value  # type: ignore[assignment,unused-ignore]
+                except (KeyError, ValueError):
+                    for decoder in self.decoders:
+                        if not isinstance(decoder, XsdWildcard):
+                            if decoder.is_matching(node.name):
+                                value = decoder.type.text_decode(node.string_value)
+                                break
+                    else:
+                        value = node.string_value
+
+            if value is None:
+                value = self.value_constraints.get(node.name)
+        else:
+            if empty:
+                value = self.value_constraints.get(None)
+
+        if value is None:
+            if not isinstance(self.field.parent, XsdKey) or \
+                    'ref' in elem.attrib and \
+                    self.field.schema.meta_schema is None and \
+                    self.field.schema.XSD_VERSION != '1.0':
+                return None
+            else:
+                msg = _("missing key field {0!r} for {1!r}")
+                raise XMLSchemaValueError(msg.format(self.field.path, self))
+        elif isinstance(value, list):
+            return tuple(value)
+        elif isinstance(value, UntypedAtomic):
+            return str(value)
+        elif isinstance(value, bool):
+            return value, bool
+        elif not isinstance(value, float):
+            return value
+        elif math.isnan(value):
+            return 'nan', float
+        else:
+            return value, float
